use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( examples lib script t xt ) );
__DATA__
Alex
Andrew
Aristotle
Berthold
Dagfinn
David
Dieter
Etheridge
Fowler
Glenn
Graham
Hans
Howarth
Ilmari
JAWNSY
Jens
Jesse
Jonathan
Karen
Knop
Kogman
Leermakers
Lowe
Luehrs
Lukas
Mai
Mannsåker
Marc
Mark
Mims
Pagaltzis
Pali
Paul
Pearcey
Peter
Rabbitson
Ricardo
Rudolf
Signes
Stosberg
Tiny
Try
Yates
Yu
Yuval
alex
anaxagoras
awalker
ayates
cebjyre
chromatic
cm
davidl
doy
ether
haarg
hdp
ilmari
irc
jens
lib
mai
marc
mark
nothingmuch
pagaltzis
pali
paul
ribasushi
rjbs
rudolf
walkeraj
יובל
קוג
